@@ -58,6 +58,54 @@
 #' @param initialTime This argument can be used to set the initial time step
 #'   shown when the map is created. It is used only when argument \code{time} is
 #'   set.
+#' @param onChange (For power users who know javascript) A character string
+#'   containing javascript code that is exexuted each time a chart is updated.
+#'   See the details section to understand why and how to use this parameter.
+#'
+#' @details
+#' Since version 0.5, the parameter \code{onChange} can be used to execute
+#' some arbitrary javascript code each time a chart is updated (with
+#' \code{updateMinicharts()} or when time step changes). A typical use case
+#' would be to change the color of a polygon added with
+#' \code{\link[leaflet]{addPolygons}} based on the data of the chart. It is even
+#' possible to create an invisible chart and use it to manage the color and the
+#' popup of a polygon. Here is a sample code that do that:
+#'
+#' \preformatted{
+#'   leaflet() \%>\% addTiles() \%>\%
+#'     addPolygons(data = myPolygons, layerId = myPolygons$myIds) \%>\%
+#'     addMinicharts(
+#'       mydata$lon, mydata$lat,
+#'       time = mydata^time
+#'       fillColor = mydata$color,
+#'       layerId = mydata$myIds,
+#'       width = 0, height = 0,
+#'       onChange = "
+#'         var s = this._map.layerManager.getLayer("shape", this.layerId);
+#'         s.bindPopup(popup);
+#'         if (opts.fillColor) {
+#'           d3.select(s._path)
+#'           .transition()
+#'           .duration(750)
+#'           .attr("fill", opts.fillColor);
+#'         }"
+#'     )
+#'
+#' }
+#'
+#' The following objects are available when executing the javascript code:
+#' \describe{
+#'   \item{this}{The current minichart object. See
+#'   \url{https://rte-antares-rpackage.github.io/leaflet.minichart/-_L.Minichart_.html}
+#'   for more information.
+#'   }
+#'   \item{opts}{The current options passed to the current minichart object.}
+#'   \item{popup}{Popup html.}
+#'   \item{d3}{The D3 module.}
+#' }
+#'
+#' Here is a toy example
+#'
 #'
 #' @return
 #' The modified leaflet map object. \code{addMinicharts} add new minicharts to
@@ -84,7 +132,7 @@ addMinicharts <- function(map, lng, lat, chartdata = 1, time = NULL, maxValues =
                           transitionTime = 750,
                           popup = popupArgs(),
                           layerId = NULL, legend = TRUE, legendPosition = "topright",
-                          timeFormat = NULL, initialTime = NULL) {
+                          timeFormat = NULL, initialTime = NULL, onChange = NULL) {
   # Prepare options
   type <- match.arg(type, c("auto", "bar", "pie", "polar-area", "polar-radius"))
   if (is.null(layerId)) layerId <- sprintf("_minichart (%s,%s)", lng, lat)
@@ -98,7 +146,7 @@ addMinicharts <- function(map, lng, lat, chartdata = 1, time = NULL, maxValues =
     labels <- "none"
   }
 
-  options <- .makeOptions(
+  options <- .preprocessArgs(
     required = list(lng = lng, lat = lat, layerId = layerId, time = time),
     optional = list(type = type, width = width, height = height,
                     opacity = opacity, labels = labels,
@@ -107,23 +155,17 @@ addMinicharts <- function(map, lng, lat, chartdata = 1, time = NULL, maxValues =
                     transitionTime = transitionTime, fillColor = fillColor)
   )
 
-  args <- .prepareArgs(options, chartdata, popup)
+  args <- .prepareJSArgs(options, chartdata, popup, onChange,
+                         initialTime = initialTime, timeFormat = timeFormat)
 
   if (is.null(maxValues)) maxValues <- args$maxValues
 
   # Add minichart and font-awesome to the map dependencies
   map$dependencies <- c(map$dependencies, minichartDeps())
 
-  # Prepare time label
-  timeLabels <- sort(unique(time))
-  if (!is.null(timeFormat)) {
-    timeLabels <- format(timeLabels, format = timeFormat)
-    if (!is.null(initialTime)) initialTime <- format(initialTime, format = timeFormat)
-  }
-
   map <- invokeMethod(map, data = leaflet::getMapData(map), "addMinicharts",
                       args$options, args$chartdata, maxValues, colorPalette,
-                      I(timeLabels), initialTime, args$popupArgs)
+                      args$timeLabels, args$initialTime, args$popupArgs, args$onChange)
 
   if (legend && length(args$legendLab) > 0 && args$ncol > 1) {
     legendCol <- colorPalette[(seq_len(args$ncols)-1) %% args$ncols + 1]
@@ -143,9 +185,11 @@ updateMinicharts <- function(map, layerId, chartdata = NULL, time = NULL, maxVal
                              labelMaxSize = NULL, labelStyle = NULL,
                              transitionTime = NULL, popup = NULL,
                              legend = TRUE, legendPosition = NULL,
-                             timeFormat = NULL, initialTime = NULL) {
+                             timeFormat = NULL, initialTime = NULL, onChange = NULL) {
 
-  type <- match.arg(type, c("auto", "bar", "pie", "polar-area", "polar-radius"))
+  if (!is.null(type)) {
+    type <- match.arg(type, c("auto", "bar", "pie", "polar-area", "polar-radius"))
+  }
   if (is.null(time)) time <- 1
   if (!is.null(chartdata) & !is.null(popup) & is.null(popup$labels)) popup$labels <- colnames(chartdata)
 
@@ -160,7 +204,7 @@ updateMinicharts <- function(map, layerId, chartdata = NULL, time = NULL, maxVal
     }
   }
 
-  options <- .makeOptions(
+  options <- .preprocessArgs(
     required = list(layerId = layerId, time = time),
     optional = list(type = type, width = width, height = height,
                     opacity = opacity, labels = labels,
@@ -170,7 +214,13 @@ updateMinicharts <- function(map, layerId, chartdata = NULL, time = NULL, maxVal
                     fillColor = fillColor)
   )
 
-  args <- .prepareArgs(options, chartdata, popup)
+  args <- .prepareJSArgs(options, chartdata, popup, onChange,
+                         initialTime = initialTime, timeFormat = timeFormat)
+
+  # Update time slider only if data is updated
+  if(is.null(chartdata)) {
+    args$timeLabels <- NULL
+  }
 
   # Update legend if required
   if (!is.null(args$chartdata)) {
@@ -183,21 +233,10 @@ updateMinicharts <- function(map, layerId, chartdata = NULL, time = NULL, maxVal
     }
   }
 
-  # Update time slider if data is updated
-  if(is.null(chartdata)) {
-    timeLabels <- NULL
-  } else {
-    timeLabels <- sort(unique(time))
-    if (!is.null(timeFormat)) {
-      timeLabels <- format(timeLabels, format = timeFormat)
-      if (!is.null(initialTime)) initialTime <- format(initialTime, format = timeFormat)
-    }
-  }
-
   map %>%
     invokeMethod(leaflet::getMapData(map), "updateMinicharts",
                  args$options, args$chartdata, unname(maxValues), colorPalette,
-                 I(timeLabels), initialTime, args$popupArgs, args$legendLab)
+                 args$timeLabels, args$initialTime, args$popupArgs, args$legendLab, args$onChange)
 }
 
 #' @rdname addMinicharts
